/// <reference types="node" />

import { Container, ContainerSettings, Context, ObjectId, Resolvable, Settings } from '@agentframework/container';
import { Constructor, IAttribute, PropertyInfo } from '@agentframework/core';
import { BlobLike, DocumentCollection, IChainable, IProfile, Model, PrimaryKey, RecordCollection } from '@agentframework/data';
import { ClassConstructor, ClassInitializer, Domain } from '@agentframework/domain';
import { Identity, IdentityProfile, NextFunction, Route, Sequence, StepFunction } from '@agentframework/utils';
import { ControllerAttribute, MiddlewareAttribute, WebApplicationSettings, WebContext, WebModule, WebRequest, WebRequestIdentityData, WebRequestIdentityProfile, WebRouter, WebRouterHandler } from '@agentframework/web';

export interface IdentityProfile {
	_id: ObjectId;
	id: number;
	displayName: string;
	displayImageURL: string;
}
export declare class SessionModel extends Model implements IChainable, WebRequestIdentityData {
	provider: string;
	providerId: string;
	authenticator: string;
	authenticatorToken: any;
	device: string;
	locale?: string;
	timezone?: string;
	currency?: string;
	theme?: string;
	account?: string;
	session?: string;
	token?: string;
	expires?: number;
	user?: string;
	roles?: Array<string>;
	groups?: Array<string>;
	profile?: WebRequestIdentityProfile;
	impersonate?: string;
	previous?: string;
	impersonated?: boolean;
	impersonatedAt?: Date;
	impersonatedFrom?: string;
	impersonatedBy?: IdentityProfile;
	impersonatedReason?: string;
	blocked?: boolean;
	blockedAt?: Date;
	blockedFrom?: string;
	blockedBy?: IdentityProfile;
	blockedReason?: string;
	deleted?: boolean;
	deletedAt?: Date;
	deletedFrom?: string;
	deletedBy?: IdentityProfile;
	deletedReason?: string;
}
declare class UserModel extends Model {
	id?: number;
	username?: string;
	displayName: string;
	displayImageURL?: string;
	displayImageFile?: string;
	signature?: any;
	email?: string;
	mobile?: string;
	roles?: Array<string>;
	groups?: Array<string>;
	identities?: any;
	settings?: any;
	timezone?: string;
	locale?: string;
	currency?: string;
	theme?: string;
	password?: string;
	passwordSalt?: string;
	paymentPassword?: string;
	paymentPasswordSalt?: string;
}
export interface EmailProvider {
	provider: string;
	domains: Array<string>;
}
export declare class WebFoundationSettings extends WebApplicationSettings {
	ADMIN_EMAIL?: string;
	TRUSTED_EMAIL_PROVIDERS?: Array<EmailProvider>;
	OWNER_NAME?: string;
	OWNER_COMPANY_NAME?: string;
	OWNER_COMPANY_CODE?: string;
	OWNER_COMPANY_ADDRESS?: Array<string>;
	OWNER_COMPANY_TELEPHONE?: string;
	OWNER_COMPANY_FAX?: string;
	OWNER_AGREEMENTS?: Array<any>;
	WEB_SESSION_EXPIRES_IN_MILLISECONDS: number;
	WEB_TOKEN_EXPIRES_IN_MILLISECONDS: number;
	WEB_INVITATION_EXPIRES_IN_MILLISECONDS: number;
	WEB_EMAIL_VERIFICATION_RESEND_INTERVAL_IN_MILLISECONDS: number;
	WEB_EMAIL_VERIFICATION_EXPIRES_IN_MILLISECONDS: number;
	WEB_SIGNIN_PATH?: string;
	WEB_SIGNIN_SUCCESS_PATH?: string;
	WEB_SIGNIN_FAILED_PATH?: string;
	WEB_ACCESS_DENIED_PATH?: string;
	WEB_SIGNOUT_PATH?: string;
	WEB_SIGNOUT_COMPLETE_PATH?: string;
	WEB_FEATURE_BACKGROUND_SYNC?: boolean;
	WEB_FEATURE_PUSH_NOTIFICATION?: boolean;
	WEB_FEATURE_PULL_REFRESH?: boolean;
	WEB_FEATURE_PULL_LOAD?: boolean;
	WEB_FEATURE_MANUAL_REFRESH?: boolean;
	WEB_FEATURE_INFINITE_SCROLL?: boolean;
	WEB_FEATURE_APP_VIEW_ENVIRONMENT_SETTINGS?: boolean;
	WEB_FEATURE_USER_LOGIN_USE_PASSWORD?: boolean;
	OIDC_AUTHENTICATION_EXPIRES_IN_MILLISECONDS: number;
	OIDC_NO_REDIRECTS_LIST?: Array<string>;
	OIDC_REDIRECT_LIST?: Array<string>;
	OIDC_GOOGLE?: boolean;
	OIDC_GOOGLE_ISSUER_URL?: string;
	OIDC_GOOGLE_CLIENT_ID?: string;
	OIDC_GOOGLE_CLIENT_SECRET?: string;
	OIDC_GOOGLE_SCOPES?: Array<string>;
	OIDC_GITLAB?: boolean;
	OIDC_GITLAB_ISSUER_URL?: string;
	OIDC_GITLAB_CLIENT_ID?: string;
	OIDC_GITLAB_CLIENT_SECRET?: string;
	OIDC_GITLAB_SCOPES?: Array<string>;
	OIDC_GITHUB?: boolean;
	OIDC_GITHUB_ISSUER_URL?: string;
	OIDC_GITHUB_CLIENT_ID?: string;
	OIDC_GITHUB_CLIENT_SECRET?: string;
	OIDC_GITHUB_SCOPES?: Array<string>;
	OIDC_WEIBO?: boolean;
	OIDC_WEIBO_ISSUER_URL?: string;
	OIDC_WEIBO_CLIENT_ID?: string;
	OIDC_WEIBO_CLIENT_SECRET?: string;
	OIDC_WEIBO_SCOPES?: Array<string>;
	CUSTOM_WECHAT_APP_NAME?: string;
	CUSTOM_WECHAT_APP_ID?: string;
	CUSTOM_WECHAT_APP_SECRET?: string;
	CUSTOM_GOOGLE_API_KEY?: string;
	CUSTOM_GOOGLE_ANALYTICS_ID?: string;
	CUSTOM_GOOGLE_API_EMAIL_ID?: string;
	CUSTOM_GOOGLE_API_PRIVATE_KEY?: string;
	CUSTOM_AWS_ACCESS_KEY_ID?: string;
	CUSTOM_AWS_SECRET_ACCESS_KEY?: string;
	CUSTOM_AWS_REGION?: string;
	CUSTOM_STRIPE_PUBLIC_KEY?: string;
	CUSTOM_STRIPE_SECRET_KEY?: string;
	CUSTOM_MAILGUN_API_KEY?: string;
	CUSTOM_KUBERNETES_API_URL?: string;
	CUSTOM_KUBERNETES_API_TOKEN?: string;
}
export interface WebReq extends WebRequest<WebFoundationSettings> {
}
export declare class CurrentUser extends UserModel {
	static [ClassInitializer](req: WebReq): Promise<UserModel>;
}
export interface WebCtx extends WebContext<WebFoundationSettings> {
}
export declare type ProviderProtocols = string | 'openid' | 'oauth' | 'oauth2' | 'bot';
export interface IExternalProviderPrimaryKey {
	provider: string;
	providerProtocol: ProviderProtocols;
	providerURL: string;
	appID: string;
}
export interface IExternalProviderProfile {
	provider: string;
	providerProtocol: ProviderProtocols;
	providerURL: string;
	providerFingerprint?: string;
	providerConfiguration?: any;
	appID: string;
	appSecret?: string;
	scope?: string;
	displayName: string;
	displayImageURL?: string;
	description?: string;
}
export interface IExternalProvider extends IExternalProviderPrimaryKey, IProfile {
	_id: ObjectId;
	provider: string;
	providerProtocol: ProviderProtocols;
	providerURL: string;
	providerFingerprint?: string;
	providerConfiguration?: any;
	appID: string;
	appSecret?: string;
	scope?: string;
	displayName: string;
	displayImageURL?: string;
	description?: string;
}
declare class ExternalProviderModel extends Model implements IExternalProvider {
	provider: string;
	providerProtocol: ProviderProtocols;
	providerURL: string;
	providerFingerprint: string;
	providerConfiguration: any;
	appID: string;
	appSecret: string;
	scope: string;
	displayName: string;
	displayImageURL: string;
	description: string;
	static toJSON(a: any): any;
	fromJSON(json: any): void;
}
declare class WebFoundationController extends Resolvable<WebFoundationSettings> {
	protected getFailureURL(ctx: WebCtx, err: Error): string;
}
export declare class WebFoundationService extends Resolvable<WebFoundationSettings> {
}
export interface IExternalUserIdentity {
	type: string;
	key?: string;
	value: string;
}
export interface IExternalUserKeyFields {
	uid: number | string;
	providerId: string;
}
export interface IExternalUserMandatoryFields extends IExternalUserKeyFields {
	uid: number | string;
	provider: string;
	providerProtocol: string;
	providerURL: string;
	providerId: string;
	providerData: any;
	userData: any;
}
export interface IExternalUserOptionalFields {
	user?: string;
	username?: string | null;
	domain?: string | null;
	domainCertified?: boolean;
	email?: string | null;
	emailCertified?: boolean;
	mobile?: string | null;
	mobileCertified?: boolean;
	displayName?: string;
	displayImageURL?: string | null;
	familyName?: string | null;
	givenName?: string | null;
	gender?: string | null;
	dob?: Date | null;
	status?: string | null;
	bio?: string | null;
	homepage?: string | null;
	locale?: string | null;
}
export interface IExternalUser extends IExternalUserMandatoryFields, IExternalUserOptionalFields {
	_id: ObjectId;
	uid: number | string;
	provider: string;
	providerProtocol: string;
	providerURL: string;
	providerId: string;
	providerData: string;
	userData: string;
	user?: string;
	username?: string | null;
	domain?: string | null;
	domainCertified?: boolean;
	email?: string | null;
	emailCertified?: boolean;
	mobile?: string | null;
	mobileCertified?: boolean;
	displayName?: string;
	displayImageURL?: string | null;
	familyName?: string | null;
	givenName?: string | null;
	gender?: string | null;
	dob?: Date | null;
	status?: string | null;
	bio?: string | null;
	homepage?: string | null;
	locale?: string | null;
	expireAt?: Date;
	createdAt: Date;
	createdFrom: string;
	createdBy: IProfile;
	updatedAt?: Date;
	updatedFrom?: string;
	updatedBy?: IProfile;
	blocked?: boolean;
	blockedAt?: Date;
	blockedFrom?: string;
	blockedBy?: IProfile;
	blockedReason?: string;
	deleted?: boolean;
	deletedAt?: Date;
	deletedFrom?: string;
	deletedBy?: IProfile;
	deletedReason?: string;
}
declare class ExternalUserModel extends Model implements IExternalUser {
	uid: number | string;
	provider: string;
	providerProtocol: ProviderProtocols;
	providerURL: string;
	providerId: string;
	providerData: string;
	userData: string;
	user: any;
	domain?: string;
	domainCertified?: boolean;
	email?: string | null;
	emailCertified?: boolean;
	mobile?: string | null;
	mobileCertified?: boolean;
	username?: string;
	displayName?: string;
	displayImageURL?: string;
	displayImageFile?: string;
	familyName?: string | null;
	givenName?: string | null;
	gender?: string | null;
	dob?: Date | null;
	status?: string | null;
	bio?: string | null;
	homepage?: string | null;
	locale?: string | null;
}
export interface IAuthenticationProfile extends IExternalUserMandatoryFields, IExternalUserOptionalFields {
	uid: number | string;
	provider: string;
	providerProtocol: string;
	providerId: string;
	providerData: any;
	userData: any;
	identities: Array<IExternalUserIdentity>;
	username: string | null;
	domain: string | null;
	domainCertified: boolean;
	email: string | null;
	emailCertified: boolean;
	mobile: string | null;
	mobileCertified: boolean;
	displayName: string;
	displayImageURL: string | null;
	familyName: string | null;
	givenName: string | null;
	gender: string | null;
	dob: Date | null;
	status: string | null;
	bio: string | null;
	homepage: string | null;
	locale: string | null;
}
export interface Ctx extends Context {
	settings: WebFoundationSettings;
}
export declare class ExternalUserService extends WebFoundationService {
	updateToken(ctx: Ctx, token: any, externalUser: ExternalUserModel): Promise<boolean>;
	connectUser(ctx: Ctx, externalUser: ExternalUserModel, user: UserModel): Promise<boolean>;
	findOrCreateExternalUser(ctx: Ctx, profile: IAuthenticationProfile): Promise<ExternalUserModel>;
	findOrCreateOrUpdateDifferenceExternalUser(ctx: Ctx, profile: IAuthenticationProfile): Promise<ExternalUserModel>;
}
declare class UserService extends WebFoundationService {
	findOneById(ctx: Ctx, id: PrimaryKey): Promise<UserModel | undefined>;
	isEmailBeenUsed(ctx: Ctx, email: string): Promise<boolean>;
	isMobileExists(ctx: Ctx, mobile: string): Promise<boolean>;
	findUserFromEmail(ctx: Ctx, email: string): Promise<UserModel | undefined>;
	findOrCreateUserFromExternalUser(ctx: Ctx, externalUser: IExternalUser): Promise<UserModel>;
}
export interface IAuthenticationToken {
	token_type: string;
	scope: string;
	expires_in: number;
	ext_expires_in: number;
	access_token: string;
	refresh_token: string;
	id_token: any | null;
}
declare abstract class OpenIdController extends WebFoundationController {
	protected readonly ExternalUsers: ExternalUserService;
	protected readonly Users: UserService;
	protected abstract discover(ctx: WebCtx): Promise<ExternalProviderModel>;
	private getAuthenticator;
	protected OnProfile(ctx: WebCtx, token: IAuthenticationToken, provider: ExternalProviderModel): Promise<IAuthenticationProfile | null>;
	protected abstract OnAuthenticated(ctx: WebCtx, externalProvider: ExternalProviderModel, token: IAuthenticationToken, externalUser: ExternalUserModel, foundUsers?: UserModel, loggedInUser?: UserModel): Promise<UserModel>;
}
export interface ISession extends WebRequestIdentityData {
	[x: string]: object | string | number | boolean | Array<string> | undefined;
}
export declare class SessionService extends WebFoundationService {
	IsValidToken(req: WebReq): boolean;
	IsValidSession(req: WebReq): Promise<boolean>;
	CreateSession(req: WebReq, payload: WebRequestIdentityData): Promise<ISession>;
	UpdateSession(req: WebReq, id: string, session: Partial<ISession>): Promise<boolean>;
	ExtendSession(req: WebReq, id: string): Promise<boolean>;
	TerminateSession(req: WebReq, id: string, reason: string): Promise<boolean>;
	private CreateNewSessionId;
	private CreateNewTokenId;
}
declare class AuthenticationService extends WebFoundationService {
	protected readonly Sessions: SessionService;
	IsAuthenticated(req: WebReq): Promise<boolean>;
	SignIn(req: WebReq, user: UserModel, allows?: Array<string>): Promise<ISession | undefined>;
	Update(req: WebReq, user: UserModel): Promise<boolean>;
	Refresh(req: WebReq): Promise<boolean>;
	SignOut(req: WebReq, reason?: string): Promise<boolean>;
}
declare enum EventType {
	SIGN_UP = "sign-up",
	SIGN_UP_FAILED = "sign-up-failed",
	SIGN_IN = "sign-in",
	SIGN_IN_FAILED = "sign-in-failed",
	SIGN_OUT = "sign-out",
	SESSION_EXPIRED = "session-expired",
	ACCESS_DENIED = "access-denied"
}
declare class EventModel extends Model {
	type: EventType;
	context: any;
}
declare class EventService extends WebFoundationService {
	getLatestEvents(ctx: Ctx): Promise<Array<EventModel>>;
	createSignInEvent(ctx: Ctx, idProvider: ExternalProviderModel, user: UserModel): Promise<EventModel | undefined>;
	createSignUpEvent(ctx: Ctx, idProvider: ExternalProviderModel, user: UserModel): Promise<EventModel | undefined>;
	createSignOutEvent(ctx: Ctx, idProvider: ExternalProviderModel): Promise<EventModel | undefined>;
}
declare abstract class OpenIdProfile {
	abstract getProfile(ctx: Ctx, token: IAuthenticationToken, provider: IExternalProvider): Promise<IAuthenticationProfile | null>;
}
export declare class ExternalProviderService extends WebFoundationService {
	findOrResolveExternalProvider(ctx: Ctx, profile: any): Promise<ExternalProviderModel>;
	protected createProviderId(profile: IExternalProviderProfile): ObjectId;
	protected discoverProviderConfiguration(ctx: Ctx, providerURL: string): Promise<any>;
}
declare abstract class OpenIdAuthController extends OpenIdController {
	protected readonly Events: EventService;
	protected readonly WebAuth: AuthenticationService;
	protected readonly OpenIdProvider: ExternalProviderService;
	protected abstract OpenIdProfile: OpenIdProfile;
	protected findOrResolveExternalProvider(ctx: Ctx, profile: any): Promise<ExternalProviderModel>;
	OnAuthenticated(ctx: WebCtx, externalProvider: ExternalProviderModel, token: IAuthenticationToken, externalUser: ExternalUserModel, foundUser?: UserModel, loggedInUser?: UserModel): Promise<UserModel>;
	OnProfile(ctx: WebCtx, token: IAuthenticationToken, provider: IExternalProvider): Promise<IAuthenticationProfile | null>;
}
export interface WeiboToken extends IAuthenticationToken {
	uid: number;
	appkey: string;
}
declare class WeiboProfileService extends OpenIdProfile {
	getProfile(domain: Ctx, token: WeiboToken, provider: IExternalProvider): Promise<IAuthenticationProfile | null>;
}
export declare class WeiboOpenIdAuthController extends OpenIdAuthController {
	protected readonly OpenIdProfile: WeiboProfileService;
	discover(ctx: WebCtx): Promise<ExternalProviderModel>;
}
declare class BitBucketProfileService extends OpenIdProfile {
	getProfile(ctx: Ctx, token: IAuthenticationToken, provider: IExternalProvider): Promise<IAuthenticationProfile | null>;
}
export declare class BitBucketOpenIdAuthController extends OpenIdAuthController {
	protected readonly OpenIdProfile: BitBucketProfileService;
	discover(ctx: WebCtx): Promise<ExternalProviderModel>;
}
export declare class BitBucketSettings extends Settings {
	OIDC_BITBUCKET?: boolean;
	OIDC_BITBUCKET_ISSUER_URL?: string;
	OIDC_BITBUCKET_CLIENT_ID?: string;
	OIDC_BITBUCKET_CLIENT_SECRET?: string;
	OIDC_BITBUCKET_SCOPES?: Array<string>;
}
export interface GoogleIdToken {
	azp: string;
	aud: string;
	sub: string;
	email: string;
	email_verified: boolean;
	at_hash: string;
	exp: number;
	iss: string;
	iat: number;
	name: string;
	picture: string;
	given_name: string;
	family_name: string;
	locale: string;
}
export interface GoogleToken extends IAuthenticationToken {
	id_token: GoogleIdToken;
}
declare class GoogleProfileService extends OpenIdProfile {
	TrustedEmailDomains: string[];
	getProfile(ctx: Ctx, token: GoogleToken, provider: IExternalProvider): Promise<IAuthenticationProfile | null>;
}
export declare class GoogleOpenIdAuthController extends OpenIdAuthController {
	protected readonly OpenIdProfile: GoogleProfileService;
	discover(ctx: WebCtx): Promise<ExternalProviderModel>;
}
declare class GitLabProfileService extends OpenIdProfile {
	getProfile(req: Ctx, token: IAuthenticationToken, provider: IExternalProvider): Promise<IAuthenticationProfile | null>;
}
export declare class GitLabOpenIdAuthController extends OpenIdAuthController {
	protected readonly OpenIdProfile: GitLabProfileService;
	discover(ctx: WebCtx): Promise<ExternalProviderModel>;
}
declare class GitHubProfileService extends OpenIdProfile {
	getProfile(req: Ctx, token: IAuthenticationToken, provider: IExternalProvider): Promise<IAuthenticationProfile | null>;
}
export declare class GitHubOpenIdAuthController extends OpenIdAuthController {
	protected readonly OpenIdProfile: GitHubProfileService;
	discover(ctx: WebCtx): Promise<ExternalProviderModel>;
}
export declare class GitHubSettings extends Settings {
	OIDC_BITBUCKET?: boolean;
	OIDC_BITBUCKET_ISSUER_URL?: string;
	OIDC_BITBUCKET_CLIENT_ID?: string;
	OIDC_BITBUCKET_CLIENT_SECRET?: string;
	OIDC_BITBUCKET_SCOPES?: Array<string>;
}
export interface MicrosoftIdToken {
	ver: string;
	iss: string;
	sub: string;
	aud: string;
	exp: number;
	iat: number;
	nbf: number;
	name: string;
	preferred_username: string;
	oid: string;
	email: string;
	tid: string;
	aio: string;
}
export interface MicrosoftToken extends IAuthenticationToken {
	id_token: MicrosoftIdToken;
}
declare class MicrosoftProfileService extends OpenIdProfile {
	TrustedMicrosoftEmailDomains: string[];
	getProfile(ctx: Ctx, token: MicrosoftToken, provider: IExternalProvider): Promise<IAuthenticationProfile | null>;
}
export declare class MicrosoftOpenIdAuthController extends OpenIdAuthController {
	protected readonly OpenIdProfile: MicrosoftProfileService;
	discover(ctx: WebCtx): Promise<ExternalProviderModel>;
}
export declare class UserCollection extends DocumentCollection<UserModel> {
	constructor();
}
export declare function dialog(name?: string): MethodDecorator;
export declare class DialogAttribute implements IAttribute {
	private _name;
	constructor(_name: string);
	beforeDecorate(target: Object, propertyKey: string | symbol, descriptor: PropertyDescriptor): boolean;
	readonly name: string;
}
declare abstract class BotService extends Resolvable<ContainerSettings> {
}
export declare abstract class BotStateService extends BotService {
	protected history: Array<[string, number, boolean?]>;
	protected onCreate(): void;
	load(...parameters: Array<any>): void;
	save(): Promise<void>;
	empty(): boolean;
	start(path: string): void;
	end(): void;
	isEnd(): boolean;
	push(path: string, stop?: boolean): void;
	pop(): [string, number, boolean?] | undefined;
	last(): [string, number, boolean?] | undefined;
	next(): any | undefined;
	restart(): void;
	clear(): void;
}
export interface BotUserProfile extends IdentityProfile {
	_id?: string;
	id?: string | number;
	name?: string;
	displayName: string;
	displayImageURL?: string;
	bot?: boolean;
}
export interface BotUser extends BotUserProfile {
	_id?: string;
	id?: string | number;
	name?: string;
	displayName: string;
	displayImageURL?: string;
	firstName?: string;
	lastName?: string;
	bot?: boolean;
	locale?: string;
	timezone?: string;
}
export interface BotEventChat {
	id: string;
	type: 'private' | 'group' | 'channel';
	name: string;
	displayName: string;
	isPrivate: boolean;
	isGroup: boolean;
	isChannel: boolean;
}
export declare abstract class BotEvent {
	abstract readonly id: string;
	abstract readonly type: string;
	abstract readonly timestamp: Date;
	abstract readonly chat: BotEventChat;
	abstract readonly sender: BotUser | undefined;
	abstract readonly provider: string;
	abstract readonly providerData: any;
	abstract readonly message: string | undefined;
	abstract readonly data: any;
	abstract readonly text: string | undefined;
}
export declare abstract class BotIdentity extends Identity {
	abstract readonly id: string;
	abstract readonly address: string;
	abstract readonly timezone: string;
	abstract readonly locale: string;
	readonly account?: string | number;
	readonly conversation?: string | number;
	readonly user?: string | number;
	readonly profile?: BotUserProfile;
}
declare abstract class BotProfile implements IProfile {
	abstract readonly id: string | number | Uint8Array;
	abstract readonly username: string;
	abstract readonly displayName: string;
	abstract readonly displayImageURL?: string;
}
export declare class BotMessage {
	readonly id: string | number | undefined;
	readonly private: boolean;
	readonly text: string | undefined;
	readonly command: string | undefined;
	readonly commandArgs: Array<string>;
}
declare class DialogAction {
}
declare class DialogRouteAction extends DialogAction {
	data: any;
	constructor(data: any);
}
declare class DialogResponseAction extends DialogAction {
}
export declare abstract class BotContext<S extends ContainerSettings = ContainerSettings> extends Container<S> {
	abstract readonly bot: BotProfile;
	abstract readonly identity: BotIdentity;
	abstract readonly event: BotEvent;
	abstract readonly message: BotMessage;
	path: string;
	pathIndex: number;
	params: any;
	sendText(text: string, user_id?: string | number): Promise<any>;
	replyText(text: string, message_id?: string | number): Promise<any>;
	startDialog(dialogPath: string, args?: any): DialogRouteAction;
	replaceDialog(dialogPath: string, args?: any): DialogRouteAction;
	restartDialog(args?: any): DialogRouteAction;
	endDialog(args?: any): DialogRouteAction;
	endConversation(args?: any): DialogRouteAction;
	resetBot(args?: any): DialogRouteAction;
}
export declare class BotStateModel {
	stack?: Array<[string, number, boolean?]>;
	state?: any;
}
export declare class StorageBackedBotStateService extends BotStateService {
	protected model: BotStateModel;
	protected id: any;
	load(filter: any, model: BotStateModel, btx: BotContext<WebFoundationSettings>): void;
	static [ClassInitializer](btx: BotContext, parameters: Array<any>): Promise<StorageBackedBotStateService>;
	save(): Promise<void>;
}
export declare class BotStateCollection extends RecordCollection<BotStateModel> {
	constructor();
}
export declare class BotSettings extends Settings {
}
declare class TypeRouterBindings extends Sequence {
	dataOf: Function;
	instance: any;
	performance: any;
	type: Constructor<any>;
	methods: Array<any>;
}
export interface DialogRouterEventHandler {
	beforeMiddleware?(controller: ControllerAttribute, middleware: MiddlewareAttribute, property: PropertyInfo): boolean;
	beforeDialog?(controller: ControllerAttribute, dialog: DialogAttribute, property: PropertyInfo): boolean;
}
declare class BotDialogRouter extends WebRouter {
	protected _dialogs: Array<any>;
	protected _root: Route;
	search(path: string): WebRouterHandler | undefined;
	add(dialog: Constructor<any>, handlers?: DialogRouterEventHandler): this;
	protected createMiddleware(type: Constructor<any>): TypeRouterBindings;
	protected addMiddleware(bindings: TypeRouterBindings, property: PropertyInfo, idx: number): void;
	protected addDialog(bindings: TypeRouterBindings, property: PropertyInfo, node: Route, idx: number): void;
	private makeInvokeMethod;
	private generateInitializerFn;
	private generateMiddlewareFn;
	private generateDialogFn;
}
declare class BotDialogEventHandler extends BotService implements DialogRouterEventHandler {
	beforeMiddleware(controller: ControllerAttribute, middleware: MiddlewareAttribute, property: PropertyInfo): boolean;
	beforeDialog(controller: ControllerAttribute, dialog: DialogAttribute, property: PropertyInfo): boolean;
	private makeMiddlewareSignature;
	private makeMethodSignature;
}
export declare class Bot<S extends ContainerSettings> extends Resolvable<S> {
	protected readonly dialog: BotDialogRouter;
	protected readonly handler: BotDialogEventHandler;
	protected readonly middleware: Sequence;
	protected onCreate(): void;
	private route;
	invoke(btx: BotContext<S>): Promise<DialogResponseAction>;
	addMiddleware(step: StepFunction): void;
	addDialog(dialog: ClassConstructor<any>): void;
	addDialogs(namespace: any): void;
}
export declare class TelegramBotController extends WebFoundationController {
	private readonly telegramBotSettings;
	private token;
	private readonly bot;
	setup(ctx: WebCtx): Promise<{
		bot: any;
		hook: any;
	}>;
	update(ctx: WebCtx): Promise<void>;
}
export declare class TelegramBot extends Bot<ContainerSettings> {
}
export declare class TelegramBotMessage extends BotMessage {
	private readonly event;
	constructor();
	readonly id: string | number | undefined;
	readonly private: boolean;
	readonly text: string | undefined;
	readonly command: string | undefined;
	readonly commandArgs: Array<string>;
}
export declare class TelegramBotIdentity extends BotIdentity {
	readonly id: string;
	readonly address: string;
	readonly timezone: string;
	readonly locale: string;
	readonly user?: string | number | undefined;
	readonly profile?: BotUserProfile | undefined;
	constructor(id: string, address: string, timezone: string, locale: string, user?: string | number | undefined, profile?: BotUserProfile | undefined);
	static [ClassInitializer](req: Container): TelegramBotIdentity;
}
export declare class TelegramBotEvent extends BotEvent {
	readonly id: string;
	readonly type: string;
	readonly timestamp: Date;
	readonly chat: BotEventChat;
	readonly sender: BotUser | undefined;
	readonly provider: string;
	readonly providerData: any;
	readonly message: string | undefined;
	readonly data: any;
	readonly text: string | undefined;
	readonly textEntities: Array<any>;
	constructor(id: string, type: string, timestamp: Date, chat: BotEventChat, sender: BotUser | undefined, provider: string, providerData: any, message: string | undefined, data: any, text: string | undefined, textEntities: Array<any>);
	static [ClassInitializer](domain: Domain, parameters: ArrayLike<any>): TelegramBotEvent;
}
export interface InitFetchOptions {
	baseURL?: string;
	headers?: any;
	query?: any;
}
export interface FetchOptions {
	method?: string;
	url?: string;
	headers?: any;
	query?: any;
	data?: any;
	raw?: boolean;
}
export interface Headers extends Iterable<[string, string]> {
	forEach(callback: (value: string, name: string) => void): void;
	append(name: string, value: string): void;
	delete(name: string): void;
	get(name: string): string | null;
	getAll(name: string): string[];
	has(name: string): boolean;
	raw(): {
		[k: string]: string[];
	};
	set(name: string, value: string): void;
	entries(): Iterator<[string, string]>;
	keys(): Iterator<string>;
	values(): Iterator<[string]>;
	[Symbol.iterator](): Iterator<[string, string]>;
}
export interface Body {
	arrayBuffer(): Promise<ArrayBufferLike>;
	body: NodeJS.ReadableStream;
	bodyUsed: boolean;
	buffer(): Promise<Buffer>;
	json(): Promise<any>;
	text(): Promise<string>;
}
export interface FetchResponse {
	url: string;
	status: number;
	statusText: string;
	type: string;
	headers: Headers;
	body: Body;
	data?: any;
}
export declare class ExternalFetcher {
	protected readonly defaults: InitFetchOptions;
	constructor(defaults: InitFetchOptions);
	private makeFetchOptions;
	protected httpRequest(options: FetchOptions): Promise<FetchResponse>;
	protected httpPost(url: string, data: any, options?: FetchOptions): Promise<FetchResponse>;
	protected httpPut(url: string, data: any, options?: FetchOptions): Promise<FetchResponse>;
	protected httpGet(url: string, options?: FetchOptions): Promise<FetchResponse>;
	protected httpGetAsText(url: string, options?: FetchOptions): Promise<string>;
	protected httpGetAsBuffer(url: string, options?: FetchOptions): Promise<Buffer>;
	protected httpGetAsJson(url: string, options?: FetchOptions): Promise<any>;
	protected httpDelete(url: string, options?: FetchOptions): Promise<FetchResponse>;
}
export interface InlineKeyboardButton {
	text: string;
	url?: string;
	callback_data?: string;
	switch_inline_query?: string;
	switch_inline_query_current_chat?: string;
	pay?: boolean;
}
export interface KeyboardButton {
	text: string;
	request_contact?: boolean;
	request_location?: boolean;
}
export declare type InlineKeyboard = Array<Array<InlineKeyboardButton>>;
export declare type Keyboard = Array<Array<KeyboardButton>>;
export interface SendTextOptions {
	chat_id?: string;
	text?: string;
	parse_mode?: string;
	disable_web_page_preview?: boolean;
	disable_notification?: boolean;
	reply_to_message_id?: number | string;
	reply_markup?: {
		inline_keyboard?: InlineKeyboard;
	} | {
		keyboard?: Keyboard;
		resize_keyboard?: boolean;
		one_time_keyboard?: boolean;
		selective?: boolean;
	} | {
		remove_keyboard?: boolean;
		selective?: boolean;
	} | {
		force_reply?: boolean;
		selective?: boolean;
	};
}
export interface EditTextOptions {
	reply_markup: {
		inline_keyboard: InlineKeyboard;
	};
}
export interface getWebhookInfoResult {
	url: string;
	has_custom_certificate: boolean;
	pending_update_count: number;
	max_connections: number;
}
export interface ResponseFromServer<T> {
	ok: 1 | 0;
	result: T;
	data: {
		ok: boolean;
		result: T;
	};
}
export interface InMemoryDocument {
	filename?: string;
	type: string;
	data: Buffer;
}
declare class TelegramClient extends ExternalFetcher {
	constructor(apiKey: any);
	static [ClassInitializer](owner: Container, params: Array<any>): any;
	getMe(): Promise<any>;
	getUserProfilePhotos(uid: any): Promise<any>;
	getFile(file_id: string): Promise<any>;
	sendMessage(chat_id: string | number, text: string, parameters?: SendTextOptions): Promise<any>;
	editMessageText(chat_id: string | number, message_id: number, text: string, parameters?: EditTextOptions): Promise<any>;
	editMessageReplyMarkup(chat_id: string | number, message_id: number, parameters?: EditTextOptions): Promise<any>;
	forwardMessage(chat_id: string | number, from_chat_id: string | number, message_id: number): Promise<any>;
	answerCallbackQuery(callback_query_id: string, text?: string, show_alert?: boolean, url?: string): Promise<any>;
	setWebhook(url: string, certificate?: string, max_connections?: number, allowed_updates?: Array<string>): Promise<any>;
	getWebhookInfo(): Promise<ResponseFromServer<getWebhookInfoResult>>;
	sendDocument(chat_id: string | number, document: InMemoryDocument, caption?: string): Promise<any>;
	sendChatAction(chat_id: string | number, action: 'typing' | 'record_video_note' | 'upload_video_note' | 'find_location' | 'upload_document' | 'upload_audio' | 'record_audio' | 'upload_video' | 'record_video' | 'upload_photo'): Promise<any>;
	sendInvoice(chat_id: string, provider_token: string, invoice: any, start_parameter: string, options?: any): Promise<any>;
	answerPreCheckoutQuery(pre_checkout_query_id: string, message?: string): Promise<any>;
	getChat(chat_id: string | number): Promise<any>;
	exportChatInviteLink(chat_id: string | number): Promise<any>;
	private post;
}
export declare class Telegram extends BotService {
	protected readonly client: TelegramClient;
	setupWebhook(url: string): Promise<any>;
	getMe(): Promise<any>;
	sendText(chat_id: string | number, text: string, parameters?: SendTextOptions): Promise<any>;
	sendInvoice(chat_id: string, provider_token: string, invoice: any, start_parameter: string): Promise<any>;
	sendCode(chat_id: string | number, text: string, parameters?: any): Promise<any>;
	confirmInvoice(invoice_id: string, message_id?: string): Promise<any>;
	getChat(chat_id: string | number): Promise<any>;
	exportChatInviteLink(chat_id: string | number): Promise<any>;
}
export declare class TelegramBotContext<S extends ContainerSettings = ContainerSettings> extends BotContext<S> {
	constructor(domain: Domain);
	readonly state: any;
	readonly bot: BotProfile;
	readonly identity: TelegramBotIdentity;
	readonly event: TelegramBotEvent;
	readonly message: TelegramBotMessage;
	readonly telegram: Telegram;
	sendText(text: string, user?: string): Promise<any>;
	sendTextWithAction(text: string, actions?: any, user?: string | number): Promise<any>;
	replyText(text: string, message_id?: string | number, user?: string | number): Promise<any>;
	sendTextWithKeyboard(): void;
}
export declare class TelegramBotSettings extends Settings {
	TELEGRAM_BOT_ID?: number;
	TELEGRAM_BOT_NAME?: string;
	TELEGRAM_BOT_DISPLAY_NAME?: string;
	TELEGRAM_BOT_TOKEN?: string;
	TELEGRAM_BOT_PAYMENT_PROVIDER_TOKEN?: string;
	TELEGRAM_BOT_SECRET_KEY?: string;
	TELEGRAM_BOT_DIALOG_TIMEOUT_IN_SECONDS?: number;
}
export declare class WechatBotController extends WebFoundationController {
	checkSender(ctx: WebCtx, next: NextFunction): Promise<any>;
	verify(ctx: WebCtx): Promise<any>;
	update(ctx: WebCtx): Promise<string>;
}
declare class FileTokenModel extends Model {
	file: string;
	type: string;
	byteLength: number;
	title?: string;
	originalname?: string;
	originalurl?: string;
}
declare class FileModel extends Model implements BlobLike {
	type: string;
	byteLength: number;
	data?: Buffer;
	width?: number;
	height?: number;
	duration?: number;
	ipfs?: string;
}
export interface WebFileMetadata {
	type?: string;
	title?: string;
	originalname?: string;
	originalurl?: string;
	width?: number;
	height?: number;
	duration?: number;
	ipfs?: string;
}
export interface WebFileReference extends WebFileMetadata {
	id: string;
	type: string;
	byteLength: number;
	url: string;
}
export declare class WebFileService {
	findTokenById(ctx: Ctx, id: string): Promise<FileTokenModel | undefined>;
	findFileById(ctx: Ctx, id: string): Promise<FileModel | undefined>;
	updateOne(ctx: Ctx, id: string, metadata: WebFileMetadata): Promise<boolean>;
	insertFromURL(ctx: Ctx, url: string, metadata?: WebFileMetadata): Promise<WebFileReference>;
	insertFromBuffer(ctx: Ctx, data: Buffer, metadata: WebFileMetadata): Promise<WebFileReference>;
}
export declare class WebFileController {
	protected readonly WebFile: WebFileService;
	token(ctx: WebCtx): Promise<void>;
}
export declare class WebFoundationModule extends WebModule {
	onInitModule(): void;
}